/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 *  Author: Miguel Angel Lopez Hernandez <miguel@gulev.org.mx>
 *
 *  Copyright 2005 Novell, Inc.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>
#include <gnome.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-lowlevel.h>
#include "em-panel-applet.h"

static gboolean
em_tray_blink_icon ()
{
        evo_tray->blink = !evo_tray->blink;
        
	if (evo_tray->newmail) {
		gtk_image_set_from_pixbuf (GTK_IMAGE (evo_tray->image), 
					   evo_tray->blink ? evo_tray->blink_icon : evo_tray->blank_icon);
		return TRUE;
	} else {
		gtk_image_set_from_pixbuf (GTK_IMAGE (evo_tray->image),
					   evo_tray->icon);
		
		gtk_tooltips_set_tip (evo_tray->tray_icon_tooltip,
				      GTK_WIDGET (evo_tray->tray_icon),
				      "Evolution new mail notification",
				      NULL);
	}
        return FALSE;
}

static DBusHandlerResult
signal_filter (DBusConnection *connection, DBusMessage *message, void *user_data)
{
	GMainLoop *loop = user_data;

	if (dbus_message_is_signal (message,
				    DBUS_INTERFACE_ORG_FREEDESKTOP_LOCAL,
				    "Disconnected"))
	{
		g_main_loop_quit (loop);
		return DBUS_HANDLER_RESULT_HANDLED;
	} else {
		if (dbus_message_is_signal (message,
					    DBUS_INTERFACE,
					    "Newmail")
		    || dbus_message_is_signal(message,
					      DBUS_INTERFACE,
					      "MessageReading")) {
			DBusError error;
			char *s;
			const char *member = dbus_message_get_member (message);
			dbus_error_init (&error);
			
			if (dbus_message_get_args (message,
						   &error,
						   DBUS_TYPE_STRING,
						   &s,
						   DBUS_TYPE_INVALID)) {
				if (strcmp (member, "Newmail") == 0 
				    && !evo_tray->newmail) {
					
					char *tooltip = g_strdup_printf ("New mail on %s", s);

					gtk_tooltips_set_tip (evo_tray->tray_icon_tooltip,
							      GTK_WIDGET (evo_tray->tray_icon),
							      tooltip, NULL);
					g_free (tooltip);
					
					evo_tray->newmail = TRUE;

					g_timeout_add (500,
						       em_tray_blink_icon,
						       evo_tray);
				} else if (strcmp (member, "MessageReading") == 0) {
					if (evo_tray->newmail) {
						evo_tray->newmail = FALSE;

						g_timeout_add (500,
							       em_tray_blink_icon,
							       evo_tray);
					}
				}

				dbus_free (s);
			} else {
				g_print("Message received, but error getting message: %s\n", error.message);
				dbus_error_free (&error);
			}
			return DBUS_HANDLER_RESULT_HANDLED;
		}
	}
	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

GdkPixbuf *
em_tray_get_image_scaled (const char *path)
{
	GdkPixbuf *pixbuf, *scaled;
	int width, height;

	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &width, &height);
 	pixbuf = gdk_pixbuf_new_from_file (path, NULL); 
	scaled = gdk_pixbuf_scale_simple (pixbuf, width, height, GDK_INTERP_BILINEAR);

	gdk_pixbuf_unref (pixbuf);

	return scaled;
}

GdkPixbuf *
em_tray_blank_icon ()
{
        GdkPixbuf *blank;
        int width, height;
        
        gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &width, &height);
        blank = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width, height);
        gdk_pixbuf_fill (blank, 0x000);
        
        return blank;
}

void
em_tray_init ()
{
	evo_tray = g_new0 (EMTrayNotify, 1);
	char *path;

	evo_tray->tray_icon = egg_tray_icon_new ("Evolution Tray Icon");
					     
	evo_tray->icon = em_tray_get_image_scaled (DEFAULT_IMG);
 	evo_tray->image = gtk_image_new_from_pixbuf (evo_tray->icon);
	evo_tray->blank_icon = em_tray_blank_icon ();
	evo_tray->blink_icon = em_tray_get_image_scaled (BLINK_IMG);
	evo_tray->tray_icon_tooltip = gtk_tooltips_new ();
	gtk_tooltips_set_tip (evo_tray->tray_icon_tooltip,
			      GTK_WIDGET (evo_tray->tray_icon),
			      "Evolution new mail notification",
			      NULL);
        evo_tray->evbox = gtk_event_box_new ();

	gtk_container_add (GTK_CONTAINER (evo_tray->evbox), evo_tray->image);
	gtk_container_add (GTK_CONTAINER (evo_tray->tray_icon), evo_tray->evbox);

	gtk_widget_show_all (GTK_WIDGET (evo_tray->tray_icon));
}

int
main (int argc, char **argv)
{
	GMainLoop *loop;
	DBusConnection *bus;
	DBusError error;
	const char *rule = g_strdup_printf ("type='signal',interface='%s'", DBUS_INTERFACE);
	
	loop = g_main_loop_new (NULL, FALSE);
	
	dbus_error_init (&error);
	bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
	if (!bus) {
		g_warning ("Failed to connect to the D-BUS daemon: %s", error.message);
		dbus_error_free (&error);
		return 1;
	}
	dbus_connection_setup_with_g_main (bus, NULL);
	
	dbus_bus_add_match (bus, rule, &error);
	dbus_connection_add_filter (bus, signal_filter, loop, NULL);
	
	gtk_init (&argc, &argv);

	icon_theme = gnome_icon_theme_new ();
	em_tray_init ();

	g_main_loop_run (loop);

	return 0;
}
